<?php

class bemutato_model extends CI_Model {
	
	private $SQL = array(
		'BEMUTATO_CIMKEK'	=> 
					'select c.id, c.cimke
					from cimke c, bemutato_cimke bc
					where c.id = bc.cimke_id
					and bc.bemutato_id = ?',
		'BEMUTATO_ID_SZERINT'	=>
					'select b.*, f.azonosito, count(k.bemutato_id) as kedvencek_szama
					from bemutato b left join kedvenc k on b.id = k.bemutato_id inner join felhasznalo f on b.felhasznalo_id = f.id
					where b.id = ?
					group by b.id',
		'KEPEK_BEMUTATOHOZ'	=>
					'select k.id, k.kepfajl, k.belyegkepfajl, k.kepurl, k.cim, k.leiras, k.hely, k.lat, k.lng, k.sorszam, k.aktiv, k.bemutato_id, k.mime_tipus 
					from kep k
					where k.bemutato_id = ?
					and k.aktiv >= ?
					order by k.sorszam',
		'BEMUTATOK_CIMKE_SZERINT_PUB'	=>
					'select b.*, f.azonosito, count(k.bemutato_id) as kedvencek_szama
					from bemutato b 
						left join kedvenc k on b.id = k.bemutato_id 
						inner join felhasznalo f on b.felhasznalo_id = f.id
						inner join (bemutato_cimke bc inner join cimke c on bc.cimke_id = c.id) on b.id = bc.bemutato_id
					where b.publikus = 1
					and c.cimke = ?
					group by b.id
					order by kedvencek_szama desc',
		'BEMUTATOK_FELHASZNALO_SZERINT_PUB'	=>
					'select b.*, f.azonosito, count(k.bemutato_id) as kedvencek_szama
					from bemutato b 
						left join kedvenc k on b.id = k.bemutato_id 
						inner join felhasznalo f on b.felhasznalo_id = f.id
					where b.publikus = 1
					and f.azonosito = ?
					group by b.id
					order by kedvencek_szama desc',
		'BEMUTATOK_FELHASZNALO_ID_SZERINT'	=>
					'select b.*, count(k.bemutato_id) as kedvencek_szama
					from bemutato b 
						left join kedvenc k on b.id = k.bemutato_id 
					where b.felhasznalo_id = ?
					group by b.id
					order by kedvencek_szama desc',
		'KEP_BLOB'	=>
					'select k.kep
					from kep k
					where k.id = ?',
		'BELYEGKEP_BLOB'	=>
					'select k.belyegkep
					from kep k
					where k.id = ?',
		'LETEZIK_E_CIMKE_BEMUTATOHOZ'	=>
					'select c.*, bc.bemutato_id 
					from cimke c left join bemutato_cimke bc on c.id = bc.cimke_id
					where c.cimke = ?
					and (bc.bemutato_id = ?
						or bc.bemutato_id is null)',
		'UJ_KEP_SORSZAM'	=>
					'select max(k.sorszam)+1 uj_sorszam 
					from kep k
					where k.bemutato_id = ?',
		'KEP_ID_SZERINT_BLOB_NELKUL'	=>
					'select k.id, k.kepfajl, k.belyegkepfajl, k.kepurl, k.cim, k.leiras, k.hely, k.lat, k.lng, k.sorszam, k.aktiv, k.bemutato_id, k.mime_tipus
					from kep k
					where k.id = ?',
		'KEP_SORSZAM_SZERINT_BLOB_NELKUL'	=>
					'select k.id, k.kepfajl, k.belyegkepfajl, k.kepurl, k.cim, k.leiras, k.hely, k.lat, k.lng, k.sorszam, k.aktiv, k.bemutato_id, k.mime_tipus
					from kep k
					where k.bemutato_id = ?
					and k.sorszam = ?',
	);
	
	public function __construct() {
		parent::__construct();
		$this->load->database();
	}
	
	public function legnepszerubb_bemutatok() {
		$query = $this->db->get('v_legnepszerubb_bemutatok');
		$result = $query->result_array();
		$result = $this->cimkezes($result);
		return $result;
	}
	
	public function legfrissebb_bemutatok() {
		$query = $this->db->get('v_legfrissebb_bemutatok');
		$result = $query->result_array();
		$result = $this->cimkezes($result);
		return $result;
	}
	
	private function bemutato_cimkek($bemutato_id) {
		return $this->db->query(
			$this->SQL['BEMUTATO_CIMKEK'],
			array(
				$bemutato_id
			)
		)
			->result_array();
	}
	
	private function cimkezes($result) {
		foreach ($result as &$bemutato) {
			$bemutato['cimkek'] = $this->bemutato_cimkek($bemutato['id']);
		}
		return $result;
	}
	
	public function cimke_felho() {
		$MIN_SZAZALEK = 80;
		$MAX_SZAZALEK = 200;
		$query = $this->db->get('v_cimkek_szama');
		$result = $query->result_array();
		$stat = $this->db
			//->select_sum('db', 'ossz')
			->select_min('db', 'min')
			->select_max('db', 'max')
			->get('v_cimkek_szama')
			->row_array();
		$min = $stat['min'];
		$max = $stat['max'];
		foreach ($result as &$cimke) {
			if ($max == $min) {
				$meret = $MIN_SZAZALEK;
			}
			else {
				$meret = ($cimke['db'] - $min) / ($max - $min) * ($MAX_SZAZALEK - $MIN_SZAZALEK) + $MIN_SZAZALEK;
			}
			$cimke['meret'] = "{$meret}%";
		}
		return $result;
	}

	public function bemutato_adatai($bemutato_id) {
		$result = $this->db->query(
			$this->SQL['BEMUTATO_ID_SZERINT'],
			array(
				$bemutato_id
			)
		)
			->row_array();
		
		$result['publikus'] = $result['publikus'] ? TRUE : FALSE;
		
		$result['cimkek'] = $this->bemutato_cimkek($result['id']);
		$cimketomb = array();
		foreach ($result['cimkek'] as $cimke) {
			$cimketomb[] = $cimke['cimke'];
		}
		$result['cimkeszoveg'] = implode(', ', $cimketomb);
		
		return $result;
	}
	
	public function bemutato_kepei($bemutato_id, $csak_aktiv = true) {
		return $this->db->query(
			$this->SQL['KEPEK_BEMUTATOHOZ'],
			array(
				$bemutato_id,
				$csak_aktiv ? 1 : 0
			)
		)
			->result_array();
	}
	
	public function bemutatok_cimke_szerint($cimke) {
		$result = $this->db->query(
			$this->SQL['BEMUTATOK_CIMKE_SZERINT_PUB'],
			array(
				$cimke
			)
		)
			->result_array();
		$result = $this->cimkezes($result);
		return $result;
	}
	
	public function bemutatok_felhasznalo_szerint($azonosito) {
		$result = $this->db->query(
			$this->SQL['BEMUTATOK_FELHASZNALO_SZERINT_PUB'],
			array(
				$azonosito
			)
		)
			->result_array();
		$result = $this->cimkezes($result);
		return $result;
	}
	
	public function bemutato_lista($felhasznalo_id) {
		$result = $this->db->query(
			$this->SQL['BEMUTATOK_FELHASZNALO_ID_SZERINT'],
			array(
				$felhasznalo_id
			)
		)
			->result_array();
		$result = $this->cimkezes($result);
		return $result;
	}
	
	public function beszur($felhasznalo_id, $cim, $leiras = null) {
		return $this->beszur_adatok(array(
			'felhasznalo_id' => $felhasznalo_id,
			'cim' => $cim,
			'leiras' => $leiras,
		));
	}
	
	public function beszur_adatok($adatok) {
		$this->db->insert('bemutato', $adatok);
		return $this->db->insert_id();
	}
	
	public function megtekintes_novel($bemutato_id) {
		$this->db->set('megtekintes_db', 'megtekintes_db + 1', FALSE);
		$this->db->where('id', $bemutato_id);
		return $this->db->update('bemutato');
	}
	
	public function modosit($bemutato_id, $cim, $leiras, $publikus, $cimkeszoveg) {
		$this->db->trans_start();

		$this->db->update(
			'bemutato',
			array(
				'cim'		=>  $cim,
				'leiras'	=>  $leiras,
				'publikus'	=>  $publikus,
			),
			array(
				'id' => $bemutato_id,
			)
		);
		
		$this->db->delete('bemutato_cimke', array(
			'bemutato_id' => $bemutato_id,
		));

		$cimkek = explode(',', $cimkeszoveg);
		foreach ($cimkek as $cimke) {
			$cimke = strtolower(trim($cimke));
			$q = $this->db->get_where('cimke', array(
				'cimke' => $cimke,
			));
			if ($q->num_rows() > 0) {
				$sor = $q->row_array();
				$cimke_id = $sor['id'];
			}
			else {
				$this->db->insert('cimke', array('cimke' => $cimke));
				$cimke_id = $this->db->insert_id();
			}
			$this->db->insert('bemutato_cimke', array(
				'bemutato_id' => $bemutato_id,
				'cimke_id' => $cimke_id,
			));
		}

		return $this->db->trans_complete();
	}
	
	public function bemutato_torol($bemutato_id) {
		return $this->db->query(
				'CALL sp_bemutato_torlese(?)', 
				array(
					$bemutato_id
				));
	}
	
	public function feltolt_kep($bemutato_id, $fajl_adatok) {
		$this->load->library('image_lib');
		
		$nagy_fajlnev = $this->atmeretezes($fajl_adatok['full_path'], 'nagy');
		$belyegkep_fajlnev = $this->atmeretezes($fajl_adatok['full_path'], 'kicsi');
		
		$nagy_fajl = file_get_contents($nagy_fajlnev);
		$belyegkep_fajl = file_get_contents($belyegkep_fajlnev);
		
		$sorszam_result = $this->db->query(
			$this->SQL['UJ_KEP_SORSZAM'],
			array($bemutato_id)
		)->row_array();
		$sorszam = $sorszam_result['uj_sorszam'];
		if (!$sorszam) {
			$sorszam = 1;
		}
		
		$result = $this->db->insert('kep', array(
			'kep' => $nagy_fajl,
			'kepfajl' => $nagy_fajlnev,
			'belyegkep' => $belyegkep_fajl,
			'belyegkepfajl' => $belyegkep_fajlnev,
			'cim' => $fajl_adatok['client_name'],
			'sorszam' => $sorszam,
			'aktiv' => 1,
			'bemutato_id' => $bemutato_id,
			'mime_tipus'	=> $fajl_adatok['file_type'],
		));
		
		return $result ? $this->db->insert_id() : FALSE;
	}
	
	private function atmeretezes($forras_fajlnev, $meret) {
		$this->image_lib->clear();
		
		switch ($meret) {
			case 'nagy':
				$width = 1024;
				$height = 768;
				break;
			case 'kicsi':
				$width = 100;
				$height = 67;
				break;
		}
		$utvonal = APPPATH . 'fajlok/' . $meret . '/';
		$belyegkep_fajlnev = $utvonal . basename($forras_fajlnev);
		
		$config = array(
			'image_library'		=> 'gd2',
			'source_image'		=> $forras_fajlnev,
			'new_image'			=> $belyegkep_fajlnev,
			'maintain_ratio'	=> TRUE,
			'width'				=> $width,
			'height'			=> $height,
		);

		$this->image_lib->initialize($config);

		if ($this->image_lib->resize()) {
			return $belyegkep_fajlnev;
		}

		return FALSE;
	}
	
	/*
	private function belyegkep_fajlnev_gen($utvonal, $fajlnev)
    {
        $tmp = $this->image_lib->explode_name($fajlnev);
        $ext = $tmp['ext'];

        mt_srand();
        $gen = md5(uniqid(mt_rand()));
        $belyegkep_fajlnev = $gen.$ext;
        $i = 1;
        while( is_file($utvonal.$belyegkep_fajlnev) )
        {
            $belyegkep_fajlnev = $gen.$i++.$ext;
        }

        return $utvonal.$belyegkep_fajlnev;
    }
	 */
	
	public function sorszam_modosit($kep_id, $mennyivel = 1) {
		$kep = $this->db->get_where('kep', array('id' => $kep_id))->row_array();
		$akt_sorszam = $kep['sorszam'];
		$uj_sorszam = $akt_sorszam + $mennyivel;
		
		$op = $uj_sorszam < $akt_sorszam ? '+' : '-';
		$mettol = $uj_sorszam < $akt_sorszam ? $uj_sorszam : $akt_sorszam + 1;
		$meddig = $uj_sorszam < $akt_sorszam ? $akt_sorszam - 1 : $uj_sorszam;
		
		$this->db->trans_start();
		
		$this->db->set('sorszam', "sorszam ${op} 1", FALSE);
		$this->db->where('sorszam >=', $mettol);
		$this->db->where('sorszam <=', $meddig);
		$this->db->update('kep');
		
		$this->db->update('kep', array('sorszam' => $uj_sorszam), array('id' => $kep_id));
		
		return $this->db->trans_complete();
	}
	
	public function aktiv_modosit($kep_id, $aktiv) {
		return $this->db->update('kep', array('aktiv' => $aktiv), array('id' => $kep_id));
	}
	
	public function kep_torol($kep_id) {
		return $this->db->delete('kep', array('id' => $kep_id));
	}
	
	public function kep_adatai($kep_id) {
		$result = $this->db->query(
			$this->SQL['KEP_ID_SZERINT_BLOB_NELKUL'],
			array(
				$kep_id
			)
		)
			->row_array();
		
		$result['aktiv'] = $result['aktiv'] ? TRUE : FALSE;
		
		return $result;
	}
	
	public function kep_adatai_sorszam($bemutato_id, $sorszam) {
		$result = $this->db->query(
			$this->SQL['KEP_SORSZAM_SZERINT_BLOB_NELKUL'],
			array(
				$bemutato_id,
				$sorszam
			)
		)
			->row_array();
		
		return $result;
	}
	
	public function kep_modosit($kep_id, $cim, $leiras, $hely, $lat, $lng, $aktiv) {
		return $this->db->update(
			'kep',
			array(
				'cim'		=>  $cim,
				'leiras'	=>  $leiras,
				'hely'		=>  $hely,
				'lat'		=>  $lat,
				'lng'		=>  $lng,
				'aktiv'		=>  $aktiv,
			),
			array(
				'id' => $kep_id,
			)
		);		
	}
	
	public function elozo_kep_sorszam($bemutato_id, $sorszam) {
		$result = $this->db
			->select_max('sorszam', 'elozo')
			->where('bemutato_id', $bemutato_id)
			->where('sorszam <', $sorszam)
			->where('aktiv', true)
			->get('kep')
			->row_array();
		return $result['elozo'];
	}
	
	public function kovetkezo_kep_sorszam($bemutato_id, $sorszam) {
		$result = $this->db
			->select_min('sorszam', 'kovetkezo')
			->where('bemutato_id', $bemutato_id)
			->where('sorszam >', $sorszam)
			->where('aktiv', true)
			->get('kep')
			->row_array();
		return $result['kovetkezo'];
	}
	
	public function kedvenc_e($felhasznalo_id, $bemutato_id) {
		return (bool)($this->db
			->from('kedvenc')
			->where('felhasznalo_id', $felhasznalo_id)
			->where('bemutato_id', $bemutato_id)
			->count_all_results());
	}
	
	public function kedvencce_tesz($felhasznalo_id, $bemutato_id) {
		return $this->db->insert('kedvenc', array(
					'felhasznalo_id' => $felhasznalo_id,
					'bemutato_id' => $bemutato_id,
		));
	}
	
	public function kedvencbol_torol($felhasznalo_id, $bemutato_id) {
		return $this->db->delete('kedvenc', array(
					'felhasznalo_id' => $felhasznalo_id,
					'bemutato_id' => $bemutato_id,
		));
	}
	
	
}
